@@ -402,7 +402,7 @@ class FilterContextPath(Path):
 
     def __str__(self) -> str:
         path_repr = str(self.path)
-        return "#" + path_repr[1:]
+        return "_" + path_repr[1:]
 
     def evaluate(self, context: FilterContext) -> object:
         matches = self.path.findall(context.extra_context)
@@ -449,3 +449,20 @@ class FunctionExtension(FilterExpression):
             return UNDEFINED
         args = [await arg.evaluate_async(context) for arg in self.args]
         return func(*args)
+
+
+class CurrentKey(FilterExpression):
+    """The key/property or index associated with the current object."""
+
+    __slots__ = ()
+
+    def evaluate(self, context: FilterContext) -> object:
+        if context.current_key is None:
+            return UNDEFINED
+        return context.current_key
+
+    async def evaluate_async(self, context: FilterContext) -> object:
+        return self.evaluate(context)
+
+
+CURRENT_KEY = CurrentKey()
