@@ -190,6 +190,40 @@ class IndexSelector(JSONPathSelector):
                     yield _match
 
 
+class KeysSelector(JSONPathSelector):
+    """Select an mapping's keys/properties."""
+
+    __slots__ = ()
+
+    def __str__(self) -> str:
+        return f"[{self.env.keys_selector_token}]"
+
+    def _keys(self, match: JSONPathMatch) -> Iterable[JSONPathMatch]:
+        if isinstance(match.obj, Mapping):
+            for i, key in enumerate(match.obj.keys()):
+                _match = JSONPathMatch(
+                    filter_context=match.filter_context(),
+                    obj=key,
+                    parent=match,
+                    parts=match.parts + (self.env.keys_selector_token, i),
+                    path=f"{match.path}[{self.env.keys_selector_token}][{i}]",
+                    root=match.root,
+                )
+                match.add_child(_match)
+                yield _match
+
+    def resolve(self, matches: Iterable[JSONPathMatch]) -> Iterable[JSONPathMatch]:
+        for match in matches:
+            yield from self._keys(match)
+
+    async def resolve_async(
+        self, matches: AsyncIterable[JSONPathMatch]
+    ) -> AsyncIterable[JSONPathMatch]:
+        async for match in matches:
+            for _match in self._keys(match):
+                yield _match
+
+
 class SliceSelector(JSONPathSelector):
     """Sequence slicing selector."""
 
@@ -409,7 +443,13 @@ class ListSelector(JSONPathSelector):
         env: JSONPathEnvironment,
         token: Token,
         items: List[
-            Union[SliceSelector, IndexSelector, PropertySelector, WildSelector]
+            Union[
+                SliceSelector,
+                KeysSelector,
+                IndexSelector,
+                PropertySelector,
+                WildSelector,
+            ]
         ],
     ) -> None:
         super().__init__(env=env, token=token)
@@ -427,6 +467,8 @@ class ListSelector(JSONPathSelector):
                 buf.append(f"'{item.name}'")
             elif isinstance(item, WildSelector):
                 buf.append("*")
+            elif isinstance(item, KeysSelector):
+                buf.append(self.env.keys_selector_token)
             else:
                 buf.append(str(item.index))
         return f"[{', '.join(buf)}]"
@@ -474,6 +516,7 @@ class Filter(JSONPathSelector):
                         current=val,
                         root=match.root,
                         extra_context=match.filter_context(),
+                        current_key=key,
                     )
                     try:
                         if self.expression.evaluate(context):
@@ -499,6 +542,7 @@ class Filter(JSONPathSelector):
                         current=obj,
                         root=match.root,
                         extra_context=match.filter_context(),
+                        current_key=i,
                     )
                     try:
                         if self.expression.evaluate(context):
@@ -528,6 +572,7 @@ class Filter(JSONPathSelector):
                         current=val,
                         root=match.root,
                         extra_context=match.filter_context(),
+                        current_key=key,
                     )
 
                     try:
@@ -556,6 +601,7 @@ class Filter(JSONPathSelector):
                         current=obj,
                         root=match.root,
                         extra_context=match.filter_context(),
+                        current_key=i,
                     )
 
                     try:
@@ -580,7 +626,7 @@ class Filter(JSONPathSelector):
 class FilterContext:
     """A filter expression context."""
 
-    __slots__ = ("current", "env", "root", "extra_context")
+    __slots__ = ("current", "current_key", "env", "root", "extra_context")
 
     def __init__(
         self,
@@ -589,11 +635,13 @@ class FilterContext:
         current: object,
         root: Union[Sequence[Any], Mapping[str, Any]],
         extra_context: Optional[Mapping[str, Any]] = None,
+        current_key: Union[str, int, None] = None,
     ) -> None:
         self.env = env
         self.current = current
         self.root = root
         self.extra_context = extra_context or {}
+        self.current_key = current_key
 
     def __str__(self) -> str:
         return (
